@@ -31,21 +31,7 @@
 #include <limits.h> /* For PATH_MAX */
 #include <openssl/sha.h>
 
-#include "glusterfs-fops.h" /* generated XDR values for FOPs */
-
-#ifndef IXDR_GET_LONG
-#define IXDR_GET_LONG(buf) ((long)IXDR_GET_U_INT32(buf))
-#endif
-#ifndef IXDR_PUT_LONG
-#define IXDR_PUT_LONG(buf, v) ((long)IXDR_PUT_INT32(buf, (long)(v)))
-#endif
-#ifndef IXDR_GET_U_LONG
-#define IXDR_GET_U_LONG(buf) ((u_long)IXDR_GET_LONG(buf))
-#endif
-#ifndef IXDR_PUT_U_LONG
-#define IXDR_PUT_U_LONG(buf, v) IXDR_PUT_LONG(buf, (long)(v))
-#endif
-
+#include "glusterfs/glusterfs-fops.h"
 #include "glusterfs/list.h"
 #include "glusterfs/locking.h"
 #include "glusterfs/logging.h"
@@ -57,6 +43,9 @@
 #define GF_YES 1
 #define GF_NO 0
 
+#define IS_ERROR(ret) ((ret) < 0)
+#define IS_SUCCESS(ret) ((ret) >= 0)
+
 #ifndef O_LARGEFILE
 /* savannah bug #20053, patch for compiling on darwin */
 #define O_LARGEFILE 0100000 /* from bits/fcntl.h */
@@ -91,7 +80,7 @@
 
 #define GLUSTERD_MAX_SNAP_NAME 255
 #define GLUSTERFS_SOCKET_LISTEN_BACKLOG 1024
-
+#define GLUSTERD_BRICK_SERVERS "cluster.brick-vol-servers"
 #define SLEN(str) (sizeof(str) - 1)
 
 #define ZR_MOUNTPOINT_OPT "mountpoint"
@@ -217,6 +206,8 @@ enum gf_internal_fop_indicator {
 #define GLUSTERFS_POSIXLK_COUNT "glusterfs.posixlk-count"
 #define GLUSTERFS_PARENT_ENTRYLK "glusterfs.parent-entrylk"
 #define GLUSTERFS_INODELK_DOM_COUNT "glusterfs.inodelk-dom-count"
+#define GLUSTERFS_INODELK_DOM_PREFIX "glusterfs.inodelk-dom-prefix"
+#define GLUSTERFS_MULTIPLE_DOM_LK_CNT_REQUESTS "glusterfs.multi-dom-lk-cnt-req"
 #define GFID_TO_PATH_KEY "glusterfs.gfid2path"
 #define GF_XATTR_STIME_PATTERN "trusted.glusterfs.*.stime"
 #define GF_XATTR_XTIME_PATTERN "trusted.glusterfs.*.xtime"
@@ -318,7 +309,6 @@ enum gf_internal_fop_indicator {
 
 #define DHT_SKIP_NON_LINKTO_UNLINK "unlink-only-if-dht-linkto-file"
 #define TIER_SKIP_NON_LINKTO_UNLINK "unlink-only-if-tier-linkto-file"
-#define TIER_LINKFILE_GFID "tier-linkfile-gfid"
 #define DHT_SKIP_OPEN_FD_UNLINK "dont-unlink-for-open-fd"
 #define DHT_IATT_IN_XDATA_KEY "dht-get-iatt-in-xattr"
 #define DHT_MODE_IN_XDATA_KEY "dht-get-mode-in-xattr"
@@ -435,7 +425,7 @@ static const char *const FOP_PRI_STRINGS[] = {"HIGH", "NORMAL", "LOW", "LEAST"};
 static inline const char *
 fop_pri_to_string(gf_fop_pri_t pri)
 {
-    if (pri < 0)
+    if (IS_ERROR(pri))
         return "UNSPEC";
 
     if (pri >= GF_FOP_PRI_MAX)
@@ -476,6 +466,8 @@ typedef struct _server_cmdline server_cmdline_t;
 #define GF_OPTION_DISABLE _gf_false
 #define GF_OPTION_DEFERRED 2
 
+typedef enum { _gf_none, _gf_memcheck, _gf_drd } gf_valgrind_tool;
+
 struct _cmd_args {
     /* basic options */
     char *volfile_server;
@@ -541,6 +533,7 @@ struct _cmd_args {
     int client_pid_set;
     unsigned uid_map_root;
     int32_t lru_limit;
+    int32_t invalidate_limit;
     int background_qlen;
     int congestion_threshold;
     char *fuse_mountopts;
@@ -567,7 +560,8 @@ struct _cmd_args {
     /* Run this process with valgrind? Might want to prevent calling
      * functions that prevent valgrind from working correctly, like
      * dlclose(). */
-    int valgrind;
+    gf_valgrind_tool vgtool;
+
     int localtime_logging;
 
     /* For the subdir mount */
@@ -587,6 +581,8 @@ struct _cmd_args {
 
     bool global_threading;
     bool brick_mux;
+
+    uint32_t fuse_dev_eperm_ratelimit_ns;
 };
 typedef struct _cmd_args cmd_args_t;
 
@@ -605,7 +601,9 @@ struct _glusterfs_graph {
                       in client multiplexed code path */
     pthread_mutex_t mutex;
     pthread_cond_t child_down_cond; /* for broadcasting CHILD_DOWN */
+    int parent_down;
     char graph_uuid[128];
+    char volume_id[GF_UUID_BUF_SIZE];
 };
 typedef struct _glusterfs_graph glusterfs_graph_t;
 
@@ -647,7 +645,7 @@ struct _glusterfs_ctx {
     glusterfs_graph_t *active;
 
     /* fuse or nfs (but not protocol/server) */
-    void *master;
+    void *primary;
 
     /* xlator implementing MOPs for centralized logging, volfile server */
     void *mgmt;
@@ -735,12 +733,21 @@ struct _glusterfs_ctx {
     } stats;
 
     struct list_head volfile_list;
-
     /* Add members to manage janitor threads for cleanup fd */
     struct list_head janitor_fds;
-    pthread_cond_t janitor_cond;
-    pthread_mutex_t janitor_lock;
+    pthread_cond_t fd_cond;
+    pthread_mutex_t fd_lock;
     pthread_t janitor;
+    /* The variable is use to save total posix xlator count */
+    uint32_t pxl_count;
+    uint32_t diskxl_count;
+
+    /* List of posix xlator use by disk thread*/
+    struct list_head diskth_xl;
+    pthread_mutex_t xl_lock;
+    pthread_cond_t xl_cond;
+    pthread_t disk_space_check;
+    char volume_id[GF_UUID_BUF_SIZE]; /* Used only in protocol/client */
 };
 typedef struct _glusterfs_ctx glusterfs_ctx_t;
 
